    .LI     OFF
;---------------------------------------------------------------------
;
; IR Repeater
;
; Generates a 38kHz carrier which remodulates the incoming IR signal.
; A status LED can optionally be connected, which blinks whenever a
; long enough IR signal is received. However this status LED has no
; knowledge about legal IR patterns because the repeater is inteded
; to be multi-protocol.
;
; Author: San Bergmans
; Date:   August 2017
;
; www.sbprojects.net
;
;---------------------------------------------------------------------

    .IN     p12f629
    .TF     irrepeater.hex,int

;---------------------------------------------------------------------
; GPIO assignment
;
; GPIO3 = IR-input
; GPIO4 = IR-LED4 output (low is on, needs PNP driving transistor)
; GPIO5 = Status LED (optional)
;---------------------------------------------------------------------

IRDET       =   GPIO3       IR Receiver input
IROUT       =   GPIO4       IR-LED output (low is on)
STATUSLED   =   GPIO5       Status LED output (low is on)

;---------------------------------------------------------------------
; RAM assignment
;---------------------------------------------------------------------

BIT_TIMER   =   $20         Used to time the IR bit length
STATUS_TMR  =   $21         Timer for the status LED

;---------------------------------------------------------------------
;  Initialize
;---------------------------------------------------------------------

RESET
    MOVLW   #%1000.0000     Select pull-ups, the rest is don't care
    MOVWF   OPTION_REG

    MOVLW   #%0000.0000     Disable all interrupts
    MOVWF   INTCON
    MOVWF   PIE1

    BSF     STATUS,RP0      Calibrate the oscillator
    CALL    $03FF
    MOVWF   OSCCAL

    MOVLW   #%0000.0111     Disable comparator mode
    MOVWF   CMCON
    BSF     STATUS,RP0      Initialize GPIO
    MOVLW   #%0000.1000     All bits are outputs, except GPIO3
    MOVWF   TRISIO
    BCF     STATUS,RP0

    MOVLW   #%0011.0000     Turn off all LED outputs
    MOVWF   GPIO
    MOVLW   #1               and prime the status LED timer
    MOVWF   STATUS_TMR        (timer may never be 0)

;---------------------------------------------------------------------
;  Main program loop
;
;  The program contains only a main loop, it has no subroutines.
;  Every loop lasts closely to 26 micro seconds (1/38kHz), regardless
;   what path is taken.
;---------------------------------------------------------------------

MAIN
    NOP                     Trim the loop to be as close to 38kHz
    NOP                      as possible
    BCF     GPIO,IROUT      Make all IR-LED outputs low (LED is off)

    MOVLW   #4              Let's waste some time while the IR LED
    MOVWF   BIT_TIMER        is off
.LOOPH
    DECFSZ  BIT_TIMER
    GOTO    .LOOPH
;   NOP                     With NOP F=37.1kHz. Without NOP F=38.5kHz

    BTFSS   GPIO,IRDET      What is the status of the IR input?
    GOTO    .ACTIVE         Low, so we're receiving something!

.IDLE       ;               The IR input is high (not receiving)
    NOP                     Make IDLE and ACTIVE equal in length
    NOP                      (1 missing GOTO, 1 output control)
    DECFSZ  STATUS_TMR      Decrement status timer
    GOTO    .DONE           Timer hasn't reached its minimum yet!
    INCF    STATUS_TMR      Stay at minimum now
    BSF     GPIO,STATUSLED  Turn status LED off
    GOTO    MAIN

.ACTIVE     ;               The IR input is low (receiving something)
    BSF     GPIO,IROUT      Make IR-LED output high (LED is on)
    INCFSZ  STATUS_TMR      Increment status timer
    GOTO    .DONE           Timer hasn't reached its maximum yet!
    DECF    STATUS_TMR      Stay at max now
    BCF     GPIO,STATUSLED  Turn status LED on
    GOTO    MAIN

.DONE
    NOP                     Make all paths equal length again
    GOTO    MAIN

;---------------------------------------------------------------------
;  Option bits
;---------------------------------------------------------------------

    .OR     $2007
;           b13-b12 Don't care about band gap voltage
;           b11-b9  Unimplemented
;           b8      Data code protection disabled
;           b7      Program memory code protection disabled
;           b6      Brown-out detection disabled
;           b5      No RESET input on GP3
;           b4      Power-up timer enabled
;           b3      Watchdog disabled
;           b2-b0   INTOSC - normal GPIO functions
    .CW     %00.0001.1000.0100
